package java.awt;

import javax.accessibility.AccessibleRole;
import java.awt.event.AdjustmentEvent;
import java.io.Serializable;
import java.awt.event.AdjustmentListener;
import javax.accessibility.AccessibleContext;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import sun.awt.ScrollPaneWheelScroller;
import java.awt.event.MouseWheelEvent;
import java.beans.Transient;
import java.awt.peer.ScrollPanePeer;
import sun.awt.SunToolkit;
import java.beans.ConstructorProperties;
import javax.accessibility.Accessible;

public class ScrollPane extends Container implements Accessible
{
    public static final int SCROLLBARS_AS_NEEDED = 0;
    public static final int SCROLLBARS_ALWAYS = 1;
    public static final int SCROLLBARS_NEVER = 2;
    private int scrollbarDisplayPolicy;
    private ScrollPaneAdjustable vAdjustable;
    private ScrollPaneAdjustable hAdjustable;
    private static final String base = "scrollpane";
    private static int nameCounter;
    private static final boolean defaultWheelScroll = true;
    private boolean wheelScrollingEnabled;
    private static final long serialVersionUID = 7956609840827222915L;
    
    private static native void initIDs();
    
    public ScrollPane() throws HeadlessException {
        this(0);
    }
    
    @ConstructorProperties({ "scrollbarDisplayPolicy" })
    public ScrollPane(int scrollbarDisplayPolicy) throws HeadlessException {
        this.wheelScrollingEnabled = true;
        GraphicsEnvironment.checkHeadless();
        this.layoutMgr = null;
        this.width = 100;
        this.height = 100;
        switch (scrollbarDisplayPolicy) {
            case 0:
            case 1:
            case 2: {
                this.scrollbarDisplayPolicy = scrollbarDisplayPolicy;
                this.vAdjustable = new ScrollPaneAdjustable(this, new ScrollPane.PeerFixer(this), 1);
                this.hAdjustable = new ScrollPaneAdjustable(this, new ScrollPane.PeerFixer(this), 0);
                this.setWheelScrollingEnabled(true);
                return;
            }
            default: {
                throw new IllegalArgumentException("illegal scrollbar display policy");
            }
        }
    }
    
    @Override
    String constructComponentName() {
        synchronized (ScrollPane.class) {
            return "scrollpane" + ScrollPane.nameCounter++;
        }
    }
    
    private void addToPanel(Component component, Object o, int n) {
        Panel panel = new Panel();
        panel.setLayout(new BorderLayout());
        panel.add(component);
        super.addImpl(panel, o, n);
        this.validate();
    }
    
    @Override
    protected final void addImpl(Component comp, Object o, int n) {
        synchronized (this.getTreeLock()) {
            if (this.getComponentCount() > 0) {
                this.remove(0);
            }
            if (n > 0) {
                throw new IllegalArgumentException("position greater than 0");
            }
            if (!SunToolkit.isLightweightOrUnknown(comp)) {
                super.addImpl(comp, o, n);
            }
            else {
                this.addToPanel(comp, o, n);
            }
        }
    }
    
    public int getScrollbarDisplayPolicy() {
        return this.scrollbarDisplayPolicy;
    }
    
    public Dimension getViewportSize() {
        Insets insets = this.getInsets();
        return new Dimension(this.width - insets.right - insets.left, this.height - insets.top - insets.bottom);
    }
    
    public int getHScrollbarHeight() {
        int hScrollbarHeight = 0;
        if (this.scrollbarDisplayPolicy != 2) {
            ScrollPanePeer scrollPanePeer = (ScrollPanePeer)this.peer;
            if (scrollPanePeer != null) {
                hScrollbarHeight = scrollPanePeer.getHScrollbarHeight();
            }
        }
        return hScrollbarHeight;
    }
    
    public int getVScrollbarWidth() {
        int vScrollbarWidth = 0;
        if (this.scrollbarDisplayPolicy != 2) {
            ScrollPanePeer scrollPanePeer = (ScrollPanePeer)this.peer;
            if (scrollPanePeer != null) {
                vScrollbarWidth = scrollPanePeer.getVScrollbarWidth();
            }
        }
        return vScrollbarWidth;
    }
    
    public Adjustable getVAdjustable() {
        return this.vAdjustable;
    }
    
    public Adjustable getHAdjustable() {
        return this.hAdjustable;
    }
    
    public void setScrollPosition(int value, int value2) {
        synchronized (this.getTreeLock()) {
            if (this.getComponentCount() == 0) {
                throw new NullPointerException("child is null");
            }
            this.hAdjustable.setValue(value);
            this.vAdjustable.setValue(value2);
        }
    }
    
    public void setScrollPosition(Point point) {
        this.setScrollPosition(point.x, point.y);
    }
    
    @Transient
    public Point getScrollPosition() {
        synchronized (this.getTreeLock()) {
            if (this.getComponentCount() == 0) {
                throw new NullPointerException("child is null");
            }
            return new Point(this.hAdjustable.getValue(), this.vAdjustable.getValue());
        }
    }
    
    @Override
    public final void setLayout(LayoutManager layoutManager) {
        throw new AWTError("ScrollPane controls layout");
    }
    
    @Override
    public void doLayout() {
        this.layout();
    }
    
    Dimension calculateChildSize() {
        Dimension size = this.getSize();
        Insets insets = this.getInsets();
        int width = size.width - insets.left * 2;
        int height = size.height - insets.top * 2;
        Dimension dimension = new Dimension(this.getComponent(0).getPreferredSize());
        int n2;
        int n;
        if (this.scrollbarDisplayPolicy == 0) {
            n2 = (dimension.height > height);
            n = (dimension.width > width);
        }
        else if (this.scrollbarDisplayPolicy == 1) {
            n = (n2 = true);
        }
        else {
            n = (n2 = false);
        }
        int vScrollbarWidth = this.getVScrollbarWidth();
        int hScrollbarHeight = this.getHScrollbarHeight();
        if (n2 != 0) {
            width -= vScrollbarWidth;
        }
        if (n != 0) {
            height -= hScrollbarHeight;
        }
        if (dimension.width < width) {
            dimension.width = width;
        }
        if (dimension.height < height) {
            dimension.height = height;
            return dimension;
        }
        return dimension;
    }
    
    @Deprecated
    @Override
    public void layout() {
        if (this.getComponentCount() == 0) {
            return;
        }
        Component component = this.getComponent(0);
        Point scrollPosition = this.getScrollPosition();
        Dimension calculateChildSize = this.calculateChildSize();
        this.getViewportSize();
        component.reshape(-scrollPosition.x, -scrollPosition.y, calculateChildSize.width, calculateChildSize.height);
        ScrollPanePeer scrollPanePeer = (ScrollPanePeer)this.peer;
        if (scrollPanePeer != null) {
            scrollPanePeer.childResized(calculateChildSize.width, calculateChildSize.height);
        }
        Dimension viewportSize = this.getViewportSize();
        this.hAdjustable.setSpan(0, calculateChildSize.width, viewportSize.width);
        this.vAdjustable.setSpan(0, calculateChildSize.height, viewportSize.height);
        return;
    }
    
    @Override
    public void printComponents(Graphics graphics) {
        if (this.getComponentCount() == 0) {
            return;
        }
        Component component = this.getComponent(0);
        Point location = component.getLocation();
        Dimension viewportSize = this.getViewportSize();
        Insets insets = this.getInsets();
        Graphics create = graphics.create();
        try {
            create.clipRect(insets.left, insets.top, viewportSize.width, viewportSize.height);
            create.translate(location.x, location.y);
            component.printAll(create);
        }
        finally {
            create.dispose();
        }
        return;
    }
    
    @Override
    public void addNotify() {
        synchronized (this.getTreeLock()) {
            int value = 0;
            int value2 = 0;
            if (this.getComponentCount() > 0) {
                value = this.vAdjustable.getValue();
                value2 = this.hAdjustable.getValue();
                this.vAdjustable.setValue(0);
                this.hAdjustable.setValue(0);
            }
            if (this.peer == null) {
                this.peer = this.getToolkit().createScrollPane(this);
            }
            super.addNotify();
            if (this.getComponentCount() > 0) {
                this.vAdjustable.setValue(value);
                this.hAdjustable.setValue(value2);
            }
        }
    }
    
    public String paramString() {
        String str;
        switch (this.scrollbarDisplayPolicy) {
            case 0: {
                str = "as-needed";
                break;
            }
            case 1: {
                str = "always";
                break;
            }
            case 2: {
                str = "never";
                break;
            }
            default: {
                str = "invalid display policy";
                break;
            }
        }
        Object o = (this.getComponentCount() > 0) ? this.getScrollPosition() : new Point(0, 0);
        Insets insets = this.getInsets();
        return super.paramString() + ",ScrollPosition=(" + o.x + "," + o.y + "),Insets=(" + insets.top + "," + insets.left + "," + insets.bottom + "," + insets.right + "),ScrollbarDisplayPolicy=" + str + ",wheelScrollingEnabled=" + this.isWheelScrollingEnabled();
    }
    
    @Override
    void autoProcessMouseWheel(MouseWheelEvent mouseWheelEvent) {
        this.processMouseWheelEvent(mouseWheelEvent);
    }
    
    @Override
    protected void processMouseWheelEvent(MouseWheelEvent e) {
        if (this.isWheelScrollingEnabled()) {
            ScrollPaneWheelScroller.handleWheelScrolling(this, e);
            e.consume();
        }
        super.processMouseWheelEvent(e);
    }
    
    protected boolean eventTypeEnabled(int n) {
        return (n == 507 && this.isWheelScrollingEnabled()) || super.eventTypeEnabled(n);
    }
    
    public void setWheelScrollingEnabled(boolean wheelScrollingEnabled) {
        this.wheelScrollingEnabled = wheelScrollingEnabled;
    }
    
    public boolean isWheelScrollingEnabled() {
        return this.wheelScrollingEnabled;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws ClassNotFoundException, IOException, HeadlessException {
        GraphicsEnvironment.checkHeadless();
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.scrollbarDisplayPolicy = fields.get("scrollbarDisplayPolicy", 0);
        this.hAdjustable = (ScrollPaneAdjustable)fields.get("hAdjustable", null);
        this.vAdjustable = (ScrollPaneAdjustable)fields.get("vAdjustable", null);
        this.wheelScrollingEnabled = fields.get("wheelScrollingEnabled", true);
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new ScrollPane.AccessibleAWTScrollPane();
        }
        return this.accessibleContext;
    }
    
    static {
        Toolkit.loadLibraries();
        if (!GraphicsEnvironment.isHeadless()) {
            initIDs();
        }
        ScrollPane.nameCounter = 0;
    }
    
    class PeerFixer implements AdjustmentListener, Serializable
    {
        private static final long serialVersionUID = 1043664721353696630L;
        private ScrollPane scroller;
        
        PeerFixer(ScrollPane scroller) {
            this.scroller = scroller;
        }
        
        @Override
        public void adjustmentValueChanged(AdjustmentEvent adjustmentEvent) {
            Adjustable adjustable = adjustmentEvent.getAdjustable();
            int value = adjustmentEvent.getValue();
            ScrollPanePeer scrollPanePeer = (ScrollPanePeer)this.scroller.peer;
            if (scrollPanePeer != null) {
                scrollPanePeer.setValue(adjustable, value);
            }
            Component component = this.scroller.getComponent(0);
            switch (adjustable.getOrientation()) {
                case 1: {
                    component.move(component.getLocation().x, -value);
                    break;
                }
                case 0: {
                    component.move(-value, component.getLocation().y);
                    break;
                }
                default: {
                    throw new IllegalArgumentException("Illegal adjustable orientation");
                }
            }
        }
    }
    
    protected class AccessibleAWTScrollPane extends Container.AccessibleAWTContainer
    {
        private static final long serialVersionUID = 6100703663886637L;
        
        @Override
        public AccessibleRole getAccessibleRole() {
            return AccessibleRole.SCROLL_PANE;
        }
    }
}
